﻿
using ProjectStormTrooper.Drawnings;
using ProjectStormTrooper.MovementStrategy;

namespace ProjectStormTrooper;


public partial class FormStormTrooper : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>

    private DrawningWarPlane? _drawningWarPlane;
    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    /// <summary>
    /// Инициализация формы
    /// </summary>


    public FormStormTrooper()
    {
        InitializeComponent();
        _strategy = null;
    }


    /// <summary>
    /// Метод прорисовки машины
    /// </summary>

    private void Draw()
    {
        if (_drawningWarPlane == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxStormTrooper.Width, pictureBoxStormTrooper.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningWarPlane.DrawTransport(gr);
        pictureBoxStormTrooper.Image = bmp;
    }
    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningWarPlane):
                _drawningWarPlane = new DrawningWarPlane(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningStormTrooper):
                _drawningWarPlane = new DrawningStormTrooper(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningWarPlane.SetPictureSize(pictureBoxStormTrooper.Width, pictureBoxStormTrooper.Height);
        _drawningWarPlane.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    /// <summary>
    /// Обработка нажатия кнопки "Создать штурмовик"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ButtonCreateStormTrooper_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningStormTrooper));

    private void ButtonCreateWarPlane(object sender, EventArgs e) => CreateObject(nameof(DrawningWarPlane));
    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningWarPlane == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningWarPlane.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningWarPlane.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningWarPlane.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningWarPlane.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }
    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningWarPlane == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableCar(_drawningWarPlane), pictureBoxStormTrooper.Width, pictureBoxStormTrooper.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}